# Copyright 2011 Tony Young
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This class specifies a Merkle-Damgard hashing algorithm like MD5 or SHA256.

define (require, exports, module) ->
    lang = require './../../lang'
    bit = require './../../math/bit'

    _MerkleDamgard = (input='') ->
        # The total length of the message.
        @_n = 0

        # Initialize first chunk.
        @_resetChunk()

        @update input

        return

    # Update the hasher with more input.
    _MerkleDamgard::update = (input) ->
        if typeof input == 'string'
            input = bit.stringToBytes input

        @_n += input.length

        for c in input
            @_chunk[@_chunkLength++] = c

            # We now have one complete chunk and can accept another chunk.
            if @_chunkLength == @constructor.blockSize
                @_update()
        @

    # Reset the current chunk.
    _MerkleDamgard::_resetChunk = ->
        # Create a new chunk.
        @_chunk = new bit.ByteArray @constructor.blockSize

        # Reset the chunk length counter.
        @_chunkLength = 0

        return

    # Update the hash object without buffering.
    _MerkleDamgard::_update = ->
        @_performRound()
        @_resetChunk()
        return

    # Calculate a _MerkleDamgard hash using the input the object was given.
    _MerkleDamgard::digest = ->
        # Copy the digester so we don't perform destructive updates on the
        # current one.
        md = @copy()

        # Return the hash bytes.
        md._digest()

    # Calculate a _MerkleDamgard hash using the input the object was given in hex
    # representation.
    _MerkleDamgard::hexdigest = ->
        h = @digest()

        hex = ''
        for v in h
            word = (v & 0xff).toString 16
            if word.length == 1
                word = '0' + word
            hex += word

        hex

    # Copy the _MerkleDamgard object.
    _MerkleDamgard::copy = ->
        md = new @constructor

        md._n = @_n
        md._chunk = new bit.ByteArray @_chunk
        md._chunkLength = @_chunkLength
        md._h = new bit.Word32Array @_h

        md

    # Perform a round of hashing. This is up to the implementer.
    _MerkleDamgard::_performRound = ->
        return

    # Internal method for converting hash to bytes. This is also up to the
    # implementer.
    _MerkleDamgard::_digest = ->
        return

    module.exports = _MerkleDamgard
