# Copyright 2011 Tony Young
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This is the exceptions module, which contains common exceptions.

define (require, exports, module) ->
    lang = require './lang'

    # The stuff in here is rather funky --- why? CoffeeScript has no shadowing,
    # so we put things in multiple functions to avoid it from automatically
    # declaring variables in outer scopes.

    _Error = if Error? then Error else (() ->
        # The base error class (if none can be found in the browser).
        Error = (message="error") ->
            @name = 'Error'
            @message = message
            return
    )()

    _RangeError = if RangeError? then RangeError else (() -> 
        # An error thrown when a numeric variable or parameter is out of its
        # valid range.
        RangeError = (message="out of range") ->
            @name = 'RangeError'
            @message = message
            return

        RangeError:: = lang.object _Error::
        RangeError::constructor = RangeError
    )()

    _TypeError = if TypeError? then TypeError else (() ->
        # An error thrown when a variable or parameter is not of a valid type.
        TypeError = (message="Type error") ->
            @name = 'TypeError'
            @message = message
            return

        TypeError:: = lang.object _Error::
        TypeError::constructor = TypeError
    )()

    # An error throw when compilation of a shader fails, or a program cannot be
    # linked.
    ShaderError = (message="Compliation error") ->
        @name = 'ShaderError'
        @message = message
        return

    ShaderError:: = lang.object _Error::
    ShaderError::constructor = ShaderError

    module.exports =
        Error               : _Error
        RangeError          : _RangeError
        TypeError           : _TypeError
        ShaderError         : ShaderError
