# Copyright 2011 Tony Young
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This is the GraphicsDevice class, a wrapper for the GL context.

define (require, exports, module) ->
    exceptions = require './../exceptions'

    # The constructor takes a GL context and wraps it for utility functions.
    GraphicsDevice = (canvas, opts) ->
        if not canvas.getContext?
            throw new exceptions.TypeError "canvas not supported"

        @gl = canvas.getContext('webgl', opts) or
              canvas.getContext('experimental-webgl', opts)
        if not @gl
            throw new exceptions.TypeError "webgl not supported"
        return

    # Compile a WebGL shader.
    GraphicsDevice::compileShader = (type, src) ->
        shader = @gl.createShader type
        @gl.shaderSource shader, src
        @gl.compileShader shader
        if not @gl.getShaderParameter shader, @gl.COMPILE_STATUS
            throw new exceptions.ShaderError @gl.getShaderInfoLog shader
        shader

    # Link a WebGL shader program.
    GraphicsDevice::linkProgram = (vShader, fShader) ->
        prog = @gl.createProgram()
        @gl.attachShader prog, vShader
        @gl.attachShader prog, fShader
        @gl.linkProgram prog
        if not @gl.getProgramParameter prog, @gl.LINK_STATUS
            throw new exceptions.ShaderError "could not link program"
        prog

    # Create a buffer and fill it with data. This will unbind any buffer in the
    # target.
    GraphicsDevice::createBuffer = (target, data, usage) ->
        buf = @gl.createBuffer()
        @gl.bindBuffer target, buf
        @gl.bufferData target, data, usage
        @gl.bindBuffer target, null
        buf

    # Create a texture and fill it with data. This will unbind any texture
    # currently bound.
    GraphicsDevice::createTexture2D = (image, levels, internalFormat, format, type) ->
        texture = @gl.createTexture()
        @gl.bindTexture @gl.TEXTURE_2D, texture
        @gl.texImage2D @gl.TEXTURE_2D, levels, internalFormat, format, type, image
        @gl.bindTexture @gl.TEXTURE_2D, null
        texture

    module.exports = GraphicsDevice
