# Copyright 2011 Tony Young
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# An arbitrary-precision integer representation. Based on
# System.Numerics.BigInteger except the backing store has a 16-bit stride.

define (require, exports, module) ->
    exceptions = require './../exceptions'

    # Use the base _BigInt implementation.
    _BigInt = require './_BigInt'

    # The BigInt class. This is the default constructor that expects a raw
    # array of data.
    BigInt = (data) ->
        @_data = data
        return

    # Construct a BigInt from a number.
    BigInt.fromNumber = (value) ->
        new BigInt _BigInt.int2bigInt value, 1, 1

    # Parse a BigInt from a string.
    BigInt.fromString = (value, base=10) ->
        new BigInt _BigInt.str2bigInt value, base, 1, 1

    # Copy the BigInt.
    BigInt::copy = ->
        new BigInt @_data

    # Add two BigInts together.
    BigInt::add = (rhs) ->
        if typeof rhs == 'number'
            new BigInt _BigInt.addInt @_data, rhs
        else
            new BigInt _BigInt.add @_data, rhs

    # Subtract two BigInts.
    BigInt::sub = (rhs) ->
        if typeof rhs == 'number'
            rhs = BigInt.fromNumber rhs
        new BigInt _BigInt.sub @_data, rhs

    # Multiply two BigInts.
    BigInt::mul = (rhs) ->
        if typeof rhs == 'number'
            rhs = BigInt.fromNumber rhs
        new BigInt _BigInt.mult @_data, rhs

    # Convert a BigInt to a string.
    BigInt::toString = (base=10) ->
        return _BigInt.bigInt2str @_data, base

    module.exports = BigInt
