# Copyright 2011 Tony Young
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

define (require, exports, module) ->
    content = require 'basilisk/content'
    GraphicsDevice = require 'basilisk/graphics/GraphicsDevice'
    Color = require 'basilisk/graphics/Color'
    Material = require 'basilisk/graphics/Material'

    window.module 'basilisk/content'

    test 'loadXhr (existing)', ->
        stop()
        content.loadXhr('./data/loadText.txt', 'text').then \
            (data) ->
                equal data, "This file should be loaded for testing loadText.", 'resolve'
                start()
            ,
            (e) ->
                ok false, 'rejected'
                start()

    test 'loadXhr (not existing)', ->
        stop()
        content.loadXhr('./data/nonExistentFile.txt', 'text').then \
            (data) ->
                ok false, 'resolved'
                start()
            ,
            (e) ->
                ok true, 'rejected'
                start()

    test 'loadShader', ->
        canvas = window.document.createElement 'canvas'
        g = new GraphicsDevice canvas

        stop()
        content.loadShader(g, g.gl.FRAGMENT_SHADER, './data/loadShader.frag').then \
            (shader) ->
                ok shader, 'loadShader'
                start()
            ,
            (e) ->
                ok false, e.name + ': ' + e.message
                start()

    test 'loadTexture2D', ->
        canvas = window.document.createElement 'canvas'
        g = new GraphicsDevice canvas

        stop()
        content.loadTexture2D(g, './data/loadTexture2D.png', 0, g.gl.RGBA, g.gl.RGBA, g.gl.UNSIGNED_INT).then \
            (texture) ->
                ok texture, 'loadTexture2D'
                start()
            ,
            (e) ->
                ok false, e.name + ': ' + e.message
                start()

    test 'loadMesh', ->
        canvas = window.document.createElement 'canvas'
        g = new GraphicsDevice canvas

        stop()
        content.loadMesh(g, './data/loadMesh.mesh').then \
            (mesh) ->
                equal mesh.vertices?, true, 'vertices'
                equal mesh.indices?, true, 'indices'
                equal mesh.stcoords?, true, 'stcoords'
                equal mesh.normals?, true, 'normals'
                equal mesh.tangents?, true, 'tangents'
                start()
            ,
            (e) ->
                ok false, e.name + ': ' + e.message
                start()

    test 'loadMaterial (with texture)', ->
        canvas = window.document.createElement 'canvas'
        g = new GraphicsDevice canvas

        stop()
        content.loadMaterial(g, './data/loadMaterial.material').then \
            (material) ->
                equal material.texture?, true, 'texture'
                material.texture = null
                deepEqual material, new Material Color.RED, Color.GREEN, Color.BLUE, Color.WHITE, 1.0
                start()
            ,
            (e) ->
                ok false, e.name + ': ' + e.message
                start()

    test 'loadMaterial (without texture)', ->
        canvas = window.document.createElement 'canvas'
        g = new GraphicsDevice canvas

        stop()
        content.loadMaterial(g, './data/loadMaterial2.material').then \
            (material) ->
                deepEqual material, new Material Color.RED, Color.GREEN, Color.BLUE, Color.WHITE, 1.0
                start()
            ,
            (e) ->
                ok false, e.name + ': ' + e.message
                start()
