# Copyright 2011 Tony Young
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

define (require, exports, module) ->
    HMAC = require 'basilisk/crypto/HMAC'

    window.module 'basilisk/crypto/HMAC'

    test 'HMAC::hexdigest (SHA1)', ->
        equal HMAC.sha1new('', '').hexdigest(), 'fbdb1d1b18aa6c08324b7d64b71fb76370690e1d', "empty string/key"
        equal HMAC.sha1new('test', 'test').hexdigest(), '0c94515c15e5095b8a87a50ba0df3bf38ed05fe6', "test/test"

    test 'HMAC::update (SHA1)', ->
        hmac = HMAC.sha1new 'test'
        hmac.update ''
        equal hmac.hexdigest(), 'fc85087452696e5bcbe3b7a71fde00e320af2cca', "first digest"
        hmac.update 'test'
        equal hmac.hexdigest(), '0c94515c15e5095b8a87a50ba0df3bf38ed05fe6', "double digest"
        hmac.update '12345678901234567890123456789012345678901234567890123456789012345'
        equal hmac.hexdigest(), 'adc28eec18ffdcc66d4eb0e29a462b9b400b1978', "digest with additional chunk"

    test 'HMAC::copy (SHA1)', ->
        orig = HMAC.sha1new '', ''
        copy = orig.copy()
        equal copy.hexdigest(), 'fbdb1d1b18aa6c08324b7d64b71fb76370690e1d', "empty string/key"
        orig.update 'something dumb'
        equal copy.hexdigest(), 'fbdb1d1b18aa6c08324b7d64b71fb76370690e1d', "empty string/key (again)"

    test 'HMAC::hexdigest (SHA256)', ->
        equal HMAC.sha256new('', '').hexdigest(), 'b613679a0814d9ec772f95d778c35fc5ff1697c493715653c6c712144292c5ad', "empty string/key"
        equal HMAC.sha256new('test', 'test').hexdigest(), '88cd2108b5347d973cf39cdf9053d7dd42704876d8c9a9bd8e2d168259d3ddf7', "test/test"

    test 'HMAC::update (SHA256)', ->
        hmac = HMAC.sha256new 'test'
        hmac.update ''
        equal hmac.hexdigest(), 'ad71148c79f21ab9eec51ea5c7dd2b668792f7c0d3534ae66b22f71c61523fb3', "first digest"
        hmac.update 'test'
        equal hmac.hexdigest(), '88cd2108b5347d973cf39cdf9053d7dd42704876d8c9a9bd8e2d168259d3ddf7', "double digest"
        hmac.update '12345678901234567890123456789012345678901234567890123456789012345'
        equal hmac.hexdigest(), '1fbb86f7436020f3ea42bcf459ae48d004d98b22d4bf74c05904783b0719836c', "digest with additional chunk"

    test 'HMAC::copy (SHA256)', ->
        orig = HMAC.sha256new '', ''
        copy = orig.copy()
        equal copy.hexdigest(), 'b613679a0814d9ec772f95d778c35fc5ff1697c493715653c6c712144292c5ad', "empty string/key"
        orig.update 'something dumb'
        equal copy.hexdigest(), 'b613679a0814d9ec772f95d778c35fc5ff1697c493715653c6c712144292c5ad', "empty string/key (again)"

    test 'HMAC::hexdigest (MD5)', ->
        equal HMAC.md5new('', '').hexdigest(), '74e6f7298a9c2d168935f58c001bad88', "empty string/key"
        equal HMAC.md5new('test', 'test').hexdigest(), 'cd4b0dcbe0f4538b979fb73664f51abe', "test/test"

    test 'HMAC::update (MD5)', ->
        hmac = HMAC.md5new 'test'
        hmac.update ''
        equal hmac.hexdigest(), '6aebb41d4366c2794687e1d9e7aee307', "first digest"
        hmac.update 'test'
        equal hmac.hexdigest(), 'cd4b0dcbe0f4538b979fb73664f51abe', "double digest"
        hmac.update '12345678901234567890123456789012345678901234567890123456789012345'
        equal hmac.hexdigest(), '2f69e80290a866826fb669da735af5ef', "digest with additional chunk"

    test 'HMAC::copy (MD5)', ->
        orig = HMAC.md5new '', ''
        copy = orig.copy()
        equal copy.hexdigest(), '74e6f7298a9c2d168935f58c001bad88', "empty string/key"
        orig.update 'something dumb'
        equal copy.hexdigest(), '74e6f7298a9c2d168935f58c001bad88', "empty string/key (again)"
