# Copyright 2011 Tony Young
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

define (require, exports, module) ->
    GraphicsDevice = require 'basilisk/graphics/GraphicsDevice'

    VERTEX_SHADER_SOURCE = 'void main() { gl_Position = vec4(0.0, 0.0, 0.0, 0.0); }'
    FRAGMENT_SHADER_SOURCE = 'void main() { gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0); }'

    window.module 'basilisk/graphics/GraphicsDevice',
        setup : () ->
            @canvas = window.document.createElement 'canvas'
            @g = new GraphicsDevice @canvas

    test 'GraphicsDevice::compileShader', ->
        vsh = @g.compileShader @g.gl.VERTEX_SHADER, VERTEX_SHADER_SOURCE
        ok vsh, "vertex shader"
        fsh = @g.compileShader @g.gl.FRAGMENT_SHADER, FRAGMENT_SHADER_SOURCE
        ok fsh, "fragment shader"

    test 'GraphicsDevice::linkProgram', ->
        vsh = @g.compileShader @g.gl.VERTEX_SHADER, VERTEX_SHADER_SOURCE
        fsh = @g.compileShader @g.gl.FRAGMENT_SHADER, FRAGMENT_SHADER_SOURCE
        prog = @g.linkProgram vsh, fsh
        ok prog, "linked program"

    test 'GraphicsDevice::createBuffer', ->
        buf = @g.createBuffer @g.gl.ARRAY_BUFFER, new Float32Array([]), @g.gl.STATIC_DRAW
        ok buf, "created buffer"

    test 'GraphicsDevice::createTexture2D', ->
        canvas = window.document.createElement 'canvas'
        canvas.width = 512
        canvas.height = 512
        texture = @g.createTexture2D canvas, 0, @g.gl.RGBA, @g.gl.RGBA, @g.gl.UNSIGNED_INT
        ok texture, "created texture"
