# Copyright 2011 Tony Young
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

define (require, exports, module) ->
    BigInt = require 'basilisk/math/BigInt'

    window.module 'basilisk/math/BigInt'

    test 'BigInt::add', ->
        lhs = BigInt.fromNumber 0
        equal lhs.copy().add(10).toString(), '10', '0 + 10'

        lhs = BigInt.fromNumber 10
        equal lhs.copy().add(0).toString(), '10', '10 + 0'
        equal lhs.copy().add(1).toString(), '11', '10 + 1'
        equal lhs.copy().add(-1).toString(), '9', '10 + -1'

        lhs = BigInt.fromNumber -1
        equal lhs.copy().add(1).toString(), '0', '-1 + 1'
        equal lhs.copy().add(2).toString(), '1', '-1 + 2'

        lhs = BigInt.fromNumber -2
        equal lhs.copy().add(1).toString(), '-1', '-2 + 1'

        lhs = BigInt.fromNumber 0xffff
        equal lhs.copy().add(0xffff).toString(16), '1FFFE', '0xffff + 0xffff (overflow)'

    test 'BigInt::sub', ->
        lhs = BigInt.fromNumber 0
        equal lhs.copy().sub(10).toString(), '-10', '0 - 10'
        equal lhs.copy().sub(-10).toString(), '10', '0 - -10'

        lhs = BigInt.fromNumber 10
        equal lhs.copy().sub(0).toString(), '10', '10 - 0'
        equal lhs.copy().sub(10).toString(), '0', '10 - 10'
        equal lhs.copy().sub(8).toString(), '2', '10 - 8'
        equal lhs.copy().sub(-8).toString(), '18', '10 - -8'

        lhs = BigInt.fromNumber -10
        equal lhs.copy().sub(-8).toString(), '-2', '-10 - -8'
        equal lhs.copy().sub(-12).toString(), '2', '-10 - -12'

        lhs = BigInt.fromNumber 0x10000
        equal lhs.copy().sub(0x1).toString(16), 'FFFF', '0x10000 - 0x1'

        lhs = BigInt.fromNumber 0x1
        equal lhs.copy().sub(0x10000).toString(16), '-FFFF', '0x1 - 0x10000'

    test 'BigInt::mul', ->
        lhs = BigInt.fromNumber 10
        equal lhs.copy().mul(0).toString(), '0', '0 * 0'
        equal lhs.copy().mul(1).toString(), '10', '10 * 1'
        equal lhs.copy().mul(-1).toString(), '-10', '10 * -1'

        lhs = BigInt.fromNumber 1
        equal lhs.copy().mul(10).toString(), '10', '1 * 10'

        lhs = BigInt.fromNumber -1
        equal lhs.copy().mul(10).toString(), '-10', '-1 * 10'

        lhs = BigInt.fromNumber 10
        equal lhs.copy().mul(10).toString(), '100', '10 * 10'

        lhs = BigInt.fromNumber 0xffff
        equal lhs.copy().mul(0xffff).toString(16), 'FFFE0001', '0xffff * 0xffff (overflow)'
