# Copyright 2011 Tony Young
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

define (require, exports, module) ->
    bit = require 'basilisk/math/bit'

    window.module 'basilisk/math/bit'

    test 'ror32', ->
        equal bit.ror32(1, 2), 1073741824
        equal bit.ror32(4294967295, 1), -1

    test 'rol32', ->
        equal bit.rol32(1, 2), 4
        equal bit.ror32(1, 1), -2147483648

    test 'stringToBytes', ->
        deepEqual bit.stringToBytes(""), new bit.ByteArray([])
        deepEqual bit.stringToBytes("A\u1242B\0\u4123C"), new bit.ByteArray([65, 18, 66, 66, 0, 65, 35, 67])

    test 'bytesToWords32BE', ->
        deepEqual bit.bytesToWords32BE([0x12, 0x34, 0x56, 0x78]), new bit.Word32Array([0x12345678])
        deepEqual bit.bytesToWords32BE([0x12, 0x34, 0x56, 0x78, 0x87, 0x65, 0x43, 0x21]), new bit.Word32Array([0x12345678, 0x87654321 << 0]) # IE gonna IE

    test 'bytesToWords32LE', ->
        deepEqual bit.bytesToWords32LE([0x12, 0x34, 0x56, 0x78]), new bit.Word32Array([0x78563412])
        deepEqual bit.bytesToWords32LE([0x12, 0x34, 0x56, 0x78, 0x87, 0x65, 0x43, 0x21]), new bit.Word32Array([0x78563412, 0x21436587])

    test 'words32BEToBytes', ->
        deepEqual bit.words32BEToBytes([0x12345678]), new bit.ByteArray([0x12, 0x34, 0x56, 0x78])
        deepEqual bit.words32BEToBytes([0x12345678, 0x87654321]), new bit.ByteArray([0x12, 0x34, 0x56, 0x78, 0x87, 0x65, 0x43, 0x21])

    test 'words32LEToBytes', ->
        deepEqual bit.words32LEToBytes([0x12345678]), new bit.ByteArray([0x78, 0x56, 0x34, 0x12])
        deepEqual bit.words32LEToBytes([0x12345678, 0x87654321]), new bit.ByteArray([0x78, 0x56, 0x34, 0x12, 0x21, 0x43, 0x65, 0x87])
