# Copyright 2011 Tony Young
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

define (require, exports, module) ->
    Vector3 = require 'basilisk/math/linalg/Vector3'
    Matrix4 = require 'basilisk/math/linalg/Matrix4'

    window.module 'basilisk/math/linalg/Vector3'

    TOLERANCE = 0.00001

    test 'new Vector3', ->
        vec = new Vector3 1, 2, 3
        equals vec.x, 1, 'x-value'
        equals vec.y, 2, 'y-value'
        equals vec.z, 3, 'z-value'

    test 'Vector3::toString', ->
        vec = new Vector3 1, 2, 3
        equals vec.toString(), 'Vector3(1, 2, 3)', 'toString'

    test 'Vector3::toArray', ->
        vec = new Vector3 1, 2, 3
        deepEqual vec.toArray(), new Vector3.Array [1, 2, 3]

    test 'Vector3::equals', ->
        vec1 = new Vector3 1, 2, 3
        vec2 = new Vector3 1, 2, 3
        equal vec1.equals(vec2), true, 'equal'
        vec2 = new Vector3 3, 4, 5
        equal vec1.equals(vec2), false, 'not equal'

    test 'Vector3::add', ->
        vec1 = new Vector3 1, 2, 3
        vec2 = new Vector3 4, 5, 6
        deepEqual vec1.add(vec2), new Vector3(5, 7, 9), 'add'

    test 'Vector3::sub', ->
        vec1 = new Vector3 1, 2, 3
        vec2 = new Vector3 6, 5, 4
        deepEqual vec1.sub(vec2), new Vector3(-5, -3, -1), 'sub'

    test 'Vector3::mul', ->
        vec1 = new Vector3 1, 2, 3
        vec2 = new Vector3 4, 5, 6
        deepEqual vec1.mul(vec2), new Vector3(4, 10, 18), 'vmul'
        vec = new Vector3 1, 2, 3
        deepEqual vec.mul(2), new Vector3(2, 4, 6), 'smul'

    test 'Vector3::vmul', ->
        vec1 = new Vector3 1, 2, 3
        vec2 = new Vector3 4, 5, 6
        deepEqual vec1.vmul(vec2), new Vector3(4, 10, 18), 'vmul'

    test 'Vector3::smul', ->
        vec = new Vector3 1, 2, 3
        deepEqual vec.smul(2), new Vector3(2, 4, 6), 'smul'

    test 'Vector3::dot', ->
        vec1 = new Vector3 1, 2, 3
        vec2 = new Vector3 4, 5, 6
        equal vec1.dot(vec2), 32, 'dot'

    test 'Vector3::cross', ->
        vec1 = new Vector3 1, 2, 3
        vec2 = new Vector3 4, 5, 6
        deepEqual vec1.cross(vec2), new Vector3(-3, 6, -3), 'cross'

    test 'Vector3::normSquared', ->
        vec = new Vector3 1, 2, 3
        equals vec.normSquared(), 14, 'normSquared'

    test 'Vector3::norm', ->
        vec = new Vector3 1, 2, 3
        equals vec.norm(), Math.sqrt 14, 'norm'

    test 'Vector3::normalize', ->
        vec = new Vector3 1, 2, 3
        vecn = vec.copy().normalize()
        norm = vec.norm()
        ok Math.abs(vecn.norm() - 1) < TOLERANCE, 'norm (floating point comparison)'
        ok Math.abs(vec.x / norm - vecn.x) < TOLERANCE, 'x-value (floating point comparison)'
        ok Math.abs(vec.y / norm - vecn.y) < TOLERANCE, 'y-value (floating point comparison)'
        ok Math.abs(vec.z / norm - vecn.z) < TOLERANCE, 'z-value (floating point comparison)'

    test 'Vector3::transform', ->
        mat = Matrix4.fromRowMajor \
            2, 5, 3, 4,
            1, 3, 2, 3,
            3, 4, 4, 1,
            0, 0, 0, 1
        
        vec = new Vector3 1, 2, 3
        
        out = vec.copy().transform(mat)
        ok Math.abs(out.x - 25) < TOLERANCE, 'x-value (floating point comparison)'
        ok Math.abs(out.y - 16) < TOLERANCE, 'y-value (floating point comparison)'
        ok Math.abs(out.z - 24) < TOLERANCE, 'z-value (floating point comparison)'

    test 'Vector3::lerp', ->
        vec1 = new Vector3 1, 2, 3
        vec2 = new Vector3 3, 4, 5
        veco = vec1.lerp vec2, 0.5
        ok Math.abs(veco.x - 2) < TOLERANCE, 'x-value (floating point comparison)'
        ok Math.abs(veco.y - 3) < TOLERANCE, 'y-value (floating point comparison)'
        ok Math.abs(veco.z - 4) < TOLERANCE, 'z-value (floating point comparison)'
